<?php

/**
 * Action : dissocier un ou tous les albums liés à un objet éditorial
 *
 * @plugin     Albums
 * @copyright  2014
 * @author     Tetue, Charles Razack
 * @licence    GPL
 * @package    SPIP\Albums\Action
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Dissocier un ou tous les albums liés à un objet éditorial
 *
 * @example
 *     ```
 *     #URL_ACTION_AUTEUR{dissocier_album, #ID_ALBUM/#OBJET/#ID_OBJET, #SELF}
 *     #URL_ACTION_AUTEUR{dissocier_album, tous/#OBJET/#ID_OBJET, #SELF}
 *     ```
 *
 * @param string|null $arg
 *     Arguments séparés par un slash `/`,
 *     sous la forme `id_album_ou_tous/objet/id_objet`
 *
 *     - id_album_ou_tous : soit l'identifiant d'un album
 *                          soit le mot-clé `tous` pour dissocier en masse
 *     - objet            : type d'objet à dissocier
 *     - id_objet         : identifiant de l'objet à dissocier
 * @return bool
 */
function action_dissocier_album_dist($arg = null) {

	// Si $arg n'est pas donné directement, le récupérer via _POST ou _GET
	if (is_null($arg)) {
		$securiser_action = charger_fonction('securiser_action', 'inc');
		$arg = $securiser_action();
	}
	$ok = false;

	// Éviter les notices si arg mal formaté
	if (substr_count($arg, '/') === 2) {
		[$id_album_ou_tous, $objet, $id_objet] = explode('/', $arg);
		$id_objet = (int) $id_objet;
		include_spip('action/editer_liens');

		// On liste tous les albums à dissocier
		$ids_albums = [];
		if ($id_album_ou_tous === 'tous') {
			$liens = objet_trouver_liens(['album' => '*'], [$objet => $id_objet]);
			$ids_albums = array_column($liens, 'id_album');
			$ids_albums = array_map('intval', $ids_albums);
		} else {
			$ids_albums = [(int) $id_album_ou_tous];
		}
		
		// On ne garde que ceux qu'on a le droit de dissocier
		$ids_albums_autorises = [];
		foreach ($ids_albums as $id_album) {
			if (autoriser('dissocier', 'album', $id_album, '', ['objet' => $objet, 'id_objet' => $id_objet])) {
				$ids_albums_autorises[] = $id_album;
			}
		}

		// Il faut qu'il reste au moins 1 autorisé à être dissocié
		if (count($ids_albums_autorises) > 0) {
			// objet_dissocier retourne le nombre de liens traités en cas de succès, sinon false
			$dissocier = objet_dissocier(['album' => $ids_albums_autorises], [$objet => $id_objet]);
			$ok = ($dissocier === count($ids_albums_autorises) ? true : false);
		}
	}

	if (!$ok) {
		spip_log("dissocier_album : erreur, ou action pas autorisée, ou arg pas compris : $arg", 'albums');
	}

	return $ok;
}
