<?php

/**
 * Gestion de l'action pour vérifier les albums insérés dans le texte d'un objet et mettre à jour les liens.
 *
 * @note
 * Inspiré de inc/marquer_doublons_doc.php du plugin Médias
 *
 * @plugin     Albums
 * @copyright  2014
 * @author     Tetue, tcharlss
 * @licence    GPL
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Vérifier les albums insérés dans le texte d'un objet et mettre à jour les liens.
 *
 * On cherche les occurences des raccourcis des modèles d'albums `<albumxx>` dans une liste de champs prédéfinie,
 * et on met à jour le champ `vu` des albums liés en fonction.
 *
 * On reprend la liste du plugin médias qui cherche dans `chapo` et `texte` par défaut.
 * Liste extensible en complétant `$GLOBALS['medias_liste_champs']`.
 *
 * @param string $objet
 *     Type d'objet
 * @param int $id_objet
 *     Identifiant de l'objet
 * @param array $data
 *     Liste complète ou partielle des couples champ => valeur de l'objet
 *     En cas de liste partielle, les couples manquants sont récupérés automatiquement
 * @param string $serveur
 *     Nom du connecteur
 * @return array
 *     Liste d'ids des albums insérés dans le texte
 */
function inc_objet_marquer_albums_inseres_dist(string $objet, int $id_objet, array $data, string $serveur = ''): array {

	$ids_albums_inseres = [];

	// Restreindre la recherche à une liste de champs prédéfinie
	// On se base sur la liste du plugin medias, qui cherche dans `chapo` et `texte` par défaut
	$champs_possibles = ($GLOBALS['medias_liste_champs'] ?? []);
	$data_traiter = array_intersect_key($data, array_flip($champs_possibles));

	// Chercher les couples champ / valeur manquants en cas de modif partielle d'un objet
	if ($champs_manquants = array_diff($champs_possibles, array_keys($data_traiter))) {
		include_spip('action/editer_objet');
		$data_manquant = (objet_lire($objet, $id_objet, ['champs' => $champs_manquants]) ?: []);
		$data_traiter = array_merge($data_traiter, $data_manquant);
	}

	if ($data_traiter) {
		include_spip('inc/texte');
		include_spip('base/abstract_sql');
		include_spip('base/objets');
		include_spip('action/editer_liens');

		// traiter_modeles() parse le texte et place les ids des modèles albums détectés
		// dans la globale `doublons_albums_inclus`
		$GLOBALS['doublons_albums_inclus'] = [];
		$texte = implode('', $data_traiter);
		$cle_objet = id_table_objet($objet);
		$desc_albums = lister_tables_objets_sql('spip_albums');
		$modeles = ($desc_albums['modeles'] ?? []);
		$contexte = [
			'objet'    => $objet,
			'id_objet' => $id_objet,
			$cle_objet => $id_objet,
		];
		traiter_modeles($texte, ['albums' => $modeles], '', '', null, $contexte);

		// On reset d'abord tous les albums liés
		objet_qualifier_liens(['album' => '*'], [$objet => $id_objet], ['vu' => 'non']);

		// Puis marquer les albums insérés, en vérifiant d'abord qu'ils existent bien
		if (count($GLOBALS['doublons_albums_inclus'])) {
			$albums_inseres = sql_allfetsel('id_album', 'spip_albums', sql_in('id_album', $GLOBALS['doublons_albums_inclus']));
			$ids_albums_inseres = array_column($albums_inseres, 'id_album');
			$ids_albums_inseres = array_map('intval', $ids_albums_inseres);
			// Créer le lien s'il n'existe pas déjà
			objet_associer(['album' => $ids_albums_inseres], [$objet => $id_objet], ['vu' => 'oui']);
			objet_qualifier_liens(['album' => $ids_albums_inseres], [$objet => $id_objet], ['vu' => 'oui']);
		}
	}

	return $ids_albums_inseres;
}
